@@ -7,7 +7,8 @@
 
 /* XML character types */
 
-STD_API void init_ctype16(void);
+STD_API int init_ctype16(void);
+STD_API void deinit_ctype16(void);
 STD_API int Toupper(int c);
 STD_API int Tolower(int c);
 
