@@ -409,7 +409,7 @@ Entity NextParameterEntity(Dtd dtd, Entity previous)
 
 ElementDefinition DefineElementN(Dtd dtd, const Char *name, int namelen, 
 				 ContentType type, Char *content,
-				 ContentParticle particle)
+				 ContentParticle particle, int declared)
 {
     ElementDefinition e;
     Char *t;
@@ -454,6 +454,8 @@ ElementDefinition DefineElementN(Dtd dtd, const Char *name, int namelen,
     e->type = type;
     e->content = content;
     e->particle = particle;
+    e->declared = declared;
+    e->has_attlist = 0;
     e->fsm = 0;
     e->nattributes = 0;
     e->nattralloc = 20;
@@ -487,7 +489,7 @@ ElementDefinition TentativelyDefineElementN(Dtd dtd,
 {
     ElementDefinition e;
 
-    if(!(e = DefineElementN(dtd, name, namelen, CT_any, 0, 0)))
+    if(!(e = DefineElementN(dtd, name, namelen, CT_any, 0, 0, 1)))
 	return 0;
 
     e->tentative = 1;
@@ -496,7 +498,7 @@ ElementDefinition TentativelyDefineElementN(Dtd dtd,
 }
 
 ElementDefinition RedefineElement(ElementDefinition e, ContentType type,
-				  Char *content, ContentParticle particle)
+				  Char *content, ContentParticle particle, int declared)
 {
 #ifdef FOR_LT
     e->eltsum->contentType = type;
@@ -506,6 +508,7 @@ ElementDefinition RedefineElement(ElementDefinition e, ContentType type,
     e->type = type;
     e->content = content;
     e->particle = particle;
+    e->declared = declared;
 
     return e;
 }
@@ -616,7 +619,8 @@ void FreeElementDefinition(ElementDefinition e)
 AttributeDefinition
     DefineAttributeN(ElementDefinition element, const Char *name, int namelen,
 		     AttributeType type, Char **allowed_values,
-		     DefaultType default_type, const Char *default_value)
+		     DefaultType default_type, const Char *default_value,
+		     int declared)
 {
 #ifdef FOR_LT
     int nav = 0;
@@ -670,6 +674,9 @@ AttributeDefinition
     a->allowed_values = allowed_values;
     a->default_type = default_type;
     a->default_value = default_value;
+    a->declared = declared;
+    if(declared)
+	element->has_attlist = 1;
     a->is_externally_declared = 0;
 
     if(a->type == AT_id && !element->id_attribute)
