@@ -131,6 +131,8 @@ struct element_definition {
     ContentType type;		/* The declared content */
     Char *content;		/* Element content */
     struct content_particle *particle;
+    int declared;		/* Was there a declaration for this? */
+    int has_attlist;		/* Were there any ATTLIST declarations? */
     FSM fsm;
     AttributeDefinition *attributes;
     int nattributes, nattralloc;
@@ -175,6 +177,7 @@ struct attribute_definition {
     Char **allowed_values;	/* List of allowed values, argv style */
     DefaultType default_type;	/* The type of the declared default */
     const Char *default_value;	/* The declared default value */
+    int declared;		/* Was there a declaration for this? */
     const Char *ns_attr_prefix;	/* Prefix it defines if a namespace attr */
     NSAttributeDefinition cached_nsdef;
     const Char *prefix, *local;
@@ -247,18 +250,20 @@ XML_API Entity NextParameterEntity(Dtd dtd, Entity previous);
 
 XML_API ElementDefinition DefineElementN(Dtd dtd, const Char *name, int namelen,
 				 ContentType type, Char *content,
-					 ContentParticle particle);
+					 ContentParticle particle,
+					 int declared);
 XML_API ElementDefinition TentativelyDefineElementN(Dtd dtd, 
 					    const Char *name, int namelen);
 XML_API ElementDefinition RedefineElement(ElementDefinition e, ContentType type,
-				  Char *content, ContentParticle particle);
+				  Char *content, ContentParticle particle,
+					  int declared);
 XML_API ElementDefinition FindElementN(Dtd dtd, const Char *name, int namelen);
 XML_API ElementDefinition NextElementDefinition(Dtd dtd, ElementDefinition previous);
 XML_API void FreeElementDefinition(ElementDefinition e);
 XML_API void FreeContentParticle(ContentParticle cp);
 
-#define DefineElement(dtd, name, type, content, particle) \
-    DefineElementN(dtd, name, Strlen(name), type, content, particle)
+#define DefineElement(dtd, name, type, content, particle, decl) \
+    DefineElementN(dtd, name, Strlen(name), type, content, particle, decl)
 #define TentativelyDefineElement(dtd, name) \
     TentativelyDefineElementN(dtd, name, Strlen(name))
 #define FindElement(dtd, name) FindElementN(dtd, name, Strlen(name))
@@ -267,15 +272,16 @@ XML_API AttributeDefinition DefineAttributeN(ElementDefinition element,
 				     const Char *name, int namelen,
 				     AttributeType type, Char **allowed_values,
 				     DefaultType default_type, 
-				     const Char *default_value);
+				     const Char *default_value,
+					     int declared);
 XML_API AttributeDefinition FindAttributeN(ElementDefinition element,
 				   const Char *name, int namelen);
 XML_API AttributeDefinition NextAttributeDefinition(ElementDefinition element,
 					    AttributeDefinition previous);
 XML_API void FreeAttributeDefinition(AttributeDefinition a);
 
-#define DefineAttribute(element, name, type, all, dt, dv) \
-    DefineAttributeN(element, name, Strlen(name), type, all, dt, dv)
+#define DefineAttribute(element, name, type, all, dt, dv, decl) \
+    DefineAttributeN(element, name, Strlen(name), type, all, dt, dv, decl)
 #define FindAttribute(element, name) \
     FindAttributeN(element, name, Strlen(name))
 
