@@ -194,8 +194,20 @@ int SourceTell(InputSource s)
 	if(s->complicated_utf8_line)
 	{
 	    /* examine earlier chars in line to see how many bytes they used */
-	    int i, c, n=0;
-	    for(i = 0; i < s->next; i++)
+	    int i, c, n;
+
+	    /* We cache the last result to avoid N^2 slowness on very
+	       long lines.  Thanks to Gait Boxman for suggesting this. */
+
+	    if(s->next < s->cached_line_char)
+	    {
+		/* Moved backwards in line; doesn't happen, I think */
+		s->cached_line_char = 0;
+		s->cached_line_byte = 0;
+	    }
+
+	    n = s->cached_line_byte;
+	    for(i = s->cached_line_char; i < s->next; i++)
 	    {
 		c = s->line[i];
 		if(c <= 0x7f)
@@ -215,6 +227,10 @@ int SourceTell(InputSource s)
 		    n += 6;
 
 	    }
+
+	    s->cached_line_char = s->next;
+	    s->cached_line_byte = n;
+
 	    return s->bytes_before_current_line + n;
 	}
 	else
@@ -430,6 +446,9 @@ static int get_translated_line1(InputSource s)
 		    goto more_bytes;
 		}
 		s->complicated_utf8_line = 1;
+		s->cached_line_char = 0;
+		s->cached_line_byte = 0;
+		
 		for(i=0; i<more; i++)
 		{
 		    int t = inbuf[nextin++];
