@@ -29,7 +29,7 @@ InputSource entity_open(Entity ent, void *arg);
 
 int verbose = 0, expand = 1, nsgml = 0,
     attr_defaults = 0, merge = 0, strict_xml = 0, tree = 0, validate = 0,
-    xml_space = 0, namespaces = 0, simple_error = 0;
+    xml_space = 0, namespaces = 0, simple_error = 0, experiment = 0;
 enum {o_unspec, o_none, o_bits, o_plain, o_can1, o_can2, o_can3} output_format = o_unspec;
 char *enc_name = 0;
 CharacterEncoding encoding = CE_unknown;
@@ -129,6 +129,9 @@ int main(int argc, char **argv)
 	    case 'z':
 		simple_error = 1;
 		break;
+	    case '.':
+		experiment = 1;
+		break;
 	    default:
 		fprintf(stderr, 
 			"usage: rxp [-abemnNsStvVx] [-o b|0|1|2|3] [-c encoding] [url]\n");
@@ -167,6 +170,12 @@ int main(int argc, char **argv)
     if(namespaces)
 	ParserSetFlag(p, XMLNamespaces, 1);
 
+    if(experiment)
+    {
+	ParserSetFlag(p, RelaxedAny, 1);
+	ParserSetFlag(p, AllowUndeclaredNSAttributes, 1);
+    }
+
     if(output_format == o_bits)
     {
 	ParserSetDtdCallback(p, dtd_cb);
@@ -189,8 +198,7 @@ int main(int argc, char **argv)
 
     if(nsgml)
     {
-	ParserSetFlag(p, XMLPiEnd, 0);
-	ParserSetFlag(p, XMLEmptyTagEnd, 0);
+	ParserSetFlag(p, XMLSyntax, 0);
 	ParserSetFlag(p, XMLPredefinedEntities, 0);
 	ParserSetFlag(p, XMLExternalIDs, 0);
 	ParserSetFlag(p, XMLMiscWFErrors, 0);
