@@ -74,6 +74,8 @@ struct _FILE16 {
 #define FILE16_read             0x01
 #define FILE16_write            0x02
 #define FILE16_close_underlying 0x04
+#define FILE16_crlf		0x08
+#define FILE16_needs_binary     0x8000
 
 static int FileRead(FILE16 *file, unsigned char *buf, int max_count);
 static int FileWrite(FILE16 *file, const unsigned char *buf, int count);
@@ -106,14 +108,38 @@ static int GzipFlush(FILE16 *file);
 #endif
 
 FILE16 *Stdin,  *Stdout, *Stderr;
+static int Stdin_open = 0, Stdout_open = 0, Stderr_open = 0;
 
-void init_stdio16(void) {
-    Stdin = MakeFILE16FromFILE(stdin, "r");
-    SetFileEncoding(Stdin, CE_ISO_8859_1);
-    Stdout = MakeFILE16FromFILE(stdout, "w");
-    SetFileEncoding(Stdout, CE_ISO_8859_1);
-    Stderr = MakeFILE16FromFILE(stderr, "w");
-    SetFileEncoding(Stderr, CE_ISO_8859_1);
+int init_stdio16(void) {
+    if(!Stdin_open)
+    {
+	if(!(Stdin = MakeFILE16FromFILE(stdin, "r")))
+	    return -1;
+	SetFileEncoding(Stdin, CE_ISO_8859_1);
+	Stdin_open = 1;
+    }
+    if(!Stdout_open)
+    {
+	if(!(Stdout = MakeFILE16FromFILE(stdout, "w")))
+	    return -1;
+	SetFileEncoding(Stdout, CE_ISO_8859_1);
+	Stdout_open = 1;
+    }
+    if(!Stderr_open)
+    {
+	if(!(Stderr = MakeFILE16FromFILE(stderr, "w")))
+	    return -1;
+	SetFileEncoding(Stderr, CE_ISO_8859_1);
+	Stderr_open = 1;
+    }
+
+    return 0;
+}
+
+void deinit_stdio16(void) {
+    if(Stdin_open) Fclose(Stdin);
+    if(Stdout_open) Fclose(Stdout);
+    if(Stderr_open) Fclose(Stderr);
 }
 
 static int ConvertASCII(const char8 *buf, int count, FILE16 *file);
@@ -129,11 +155,14 @@ static int ConvertUTF16(const char16 *buf, int count, FILE16 *file);
 
 static int ConvertASCII(const char8 *buf, int count, FILE16 *file)
 {
-    unsigned char outbuf[BufferSize*2];
+    /* It *could* be that every character is linefeed... */
+    unsigned char outbuf[BufferSize*4];
+    unsigned char c;
     int i, j;
 
     switch(file->enc)
     {
+    case CE_ISO_646:
     case CE_ISO_8859_1:
     case CE_ISO_8859_2:
     case CE_ISO_8859_3:
@@ -144,12 +173,26 @@ static int ConvertASCII(const char8 *buf, int count, FILE16 *file)
     case CE_ISO_8859_8:
     case CE_ISO_8859_9:
     case CE_unspecified_ascii_superset:
-	return file->write(file, (unsigned char *)buf, count);
+	if(file->flags & FILE16_crlf)
+	{
+	    for(i=j=0; i<count; i++)
+	    {
+		c = buf[i];
+		if(c == '\n')
+		    outbuf[j++] = '\r';
+		outbuf[j++] = c;
+	    }
+	    return file->write(file, outbuf, j);
+	}
+	else
+	    return file->write(file, (unsigned char *)buf, count);
 
     case CE_UTF_8:
 	for(i=j=0; i<count; i++)
 	{
-	    unsigned char c = buf[i];
+	    c = buf[i];
+	    if(c == '\n' && (file->flags & FILE16_crlf))
+		outbuf[j++] = '\r';
 	    if(c < 128)
 		outbuf[j++] = c;
 	    else
@@ -164,19 +207,31 @@ static int ConvertASCII(const char8 *buf, int count, FILE16 *file)
     case CE_ISO_10646_UCS_2B:
 	for(i=j=0; i<count; i++)
 	{
+	    c = buf[i];
+	    if(c == '\n' && (file->flags & FILE16_crlf))
+	    {
+		outbuf[j++] = 0;
+		outbuf[j++] = '\r';
+	    }
 	    outbuf[j++] = 0;
-	    outbuf[j++] = buf[i];
+	    outbuf[j++] = c;
 	}
-	return file->write(file, outbuf, count*2);
+	return file->write(file, outbuf, j);
 
     case CE_UTF_16L:
     case CE_ISO_10646_UCS_2L:
 	for(i=j=0; i<count; i++)
 	{
-	    outbuf[j++] = buf[i];
+	    c = buf[i];
+	    if(c == '\n' && (file->flags & FILE16_crlf))
+	    {
+		outbuf[j++] = '\r';
+		outbuf[j++] = 0;
+	    }
+	    outbuf[j++] = c;
 	    outbuf[j++] = 0;
 	}
-	return file->write(file, outbuf, count*2);
+	return file->write(file, outbuf, j);
 
     default:
       ERR2("Bad output character encoding %d (%s)\n",
@@ -192,24 +247,29 @@ static int ConvertASCII(const char8 *buf, int count, FILE16 *file)
 
 static int ConvertUTF16(const char16 *buf, int count, FILE16 *file)
 {
-    /* size is max for UTF-8 with saved first half */
-    unsigned char outbuf[BufferSize*3 + 1];
+    /* It *could* be that every character is linefeed... */
+    unsigned char outbuf[BufferSize*4];
+    char16 c;
     int i, j, tablenum, max;
     char8 *from_unicode;
     char32 big;
 
     switch(file->enc)
     {
+    case CE_ISO_646:		/* should really check for >127 */
     case CE_ISO_8859_1:
     case CE_unspecified_ascii_superset:
-	for(i=0; i<count; i++)
+	for(i=j=0; i<count; i++)
 	{
-	    if(buf[i] < 256)
-		outbuf[i] = (unsigned char)buf[i];
+	    c = buf[i];
+	    if(c == '\n' && (file->flags & FILE16_crlf))
+		outbuf[j++] = '\r';
+	    if(c < 256)
+		outbuf[j++] = (unsigned char)c;
 	    else
-		outbuf[i] = '?';
+		outbuf[j++] = '?';
 	}
-	return file->write(file, outbuf, count);
+	return file->write(file, outbuf, j);
 
     case CE_ISO_8859_2:
     case CE_ISO_8859_3:
@@ -222,31 +282,37 @@ static int ConvertUTF16(const char16 *buf, int count, FILE16 *file)
 	tablenum = (file->enc - CE_ISO_8859_2);
 	max = iso_max_val[tablenum];
 	from_unicode = unicode_to_iso[tablenum];
-	for(i=0; i<count; i++)
+	for(i=j=0; i<count; i++)
 	{
-	    if(buf[i] <= max)
-		outbuf[i] = (unsigned char)from_unicode[buf[i]];
+	    c = buf[i];
+	    if(c == '\n' && (file->flags & FILE16_crlf))
+		outbuf[j++] = '\r';
+	    if(c <= max)
+		outbuf[j++] = (unsigned char)from_unicode[c];
 	    else
-		outbuf[i] = '?';
+		outbuf[j++] = '?';
 	}
-	return file->write(file, outbuf, count);
+	return file->write(file, outbuf, j);
 
     case CE_UTF_8:
 	for(i=j=0; i<count; i++)
 	{
-	    if(buf[i] < 0x80)
-		outbuf[j++] = (unsigned char)buf[i];
-	    else if(buf[i] < 0x800)
+	    c = buf[i];
+	    if(c == '\n' && (file->flags & FILE16_crlf))
+		outbuf[j++] = '\r';
+	    if(c < 0x80)
+		outbuf[j++] = (unsigned char)c;
+	    else if(c < 0x800)
 	    {
-		outbuf[j++] = 0xc0 + (buf[i] >> 6);
-		outbuf[j++] = 0x80 + (buf[i] & 0x3f);
+		outbuf[j++] = 0xc0 + (c >> 6);
+		outbuf[j++] = 0x80 + (c & 0x3f);
 	    }
-	    else if(buf[i] >= 0xd800 && buf[i] <= 0xdbff)
-		file->save = buf[i];
-	    else if(buf[i] >= 0xdc00 && buf[i] <= 0xdfff)
+	    else if(c >= 0xd800 && c <= 0xdbff)
+		file->save = c;
+	    else if(c >= 0xdc00 && c <= 0xdfff)
 	    {
 		big = 0x10000 + 
-		    ((file->save - 0xd800) << 10) + (buf[i] - 0xdc00);
+		    ((file->save - 0xd800) << 10) + (c - 0xdc00);
 		outbuf[j++] = 0xf0 + (big >> 18);
 		outbuf[j++] = 0x80 + ((big >> 12) & 0x3f);
 		outbuf[j++] = 0x80 + ((big >> 6) & 0x3f);
@@ -254,9 +320,9 @@ static int ConvertUTF16(const char16 *buf, int count, FILE16 *file)
 	    }
 	    else
 	    {
-		outbuf[j++] = 0xe0 + (buf[i] >> 12);
-		outbuf[j++] = 0x80 + ((buf[i] >> 6) & 0x3f);
-		outbuf[j++] = 0x80 + (buf[i] & 0x3f);
+		outbuf[j++] = 0xe0 + (c >> 12);
+		outbuf[j++] = 0x80 + ((c >> 6) & 0x3f);
+		outbuf[j++] = 0x80 + (c & 0x3f);
 	    }
 	}
 	return file->write(file, outbuf, j);
@@ -265,21 +331,33 @@ static int ConvertUTF16(const char16 *buf, int count, FILE16 *file)
     case CE_ISO_10646_UCS_2B:
  	for(i=j=0; i<count; i++)
 	{
-	    outbuf[j++] = (buf[i] >> 8);
-	    outbuf[j++] = (buf[i] & 0xff);
+	    c = buf[i];
+	    if(c == '\n' && (file->flags & FILE16_crlf))
+	    {
+		outbuf[j++] = 0;
+		outbuf[j++] = '\r';
+	    }
+	    outbuf[j++] = (c >> 8);
+	    outbuf[j++] = (c & 0xff);
 
 	}
-	return file->write(file, outbuf, count*2);
+	return file->write(file, outbuf, j);
 
     case CE_UTF_16L:
     case CE_ISO_10646_UCS_2L:
  	for(i=j=0; i<count; i++)
 	{
-	    outbuf[j++] = (buf[i] & 0xff);
-	    outbuf[j++] = (buf[i] >> 8);
+	    c = buf[i];
+	    if(c == '\n' && (file->flags & FILE16_crlf))
+	    {
+		outbuf[j++] = '\r';
+		outbuf[j++] = 0;
+	    }
+	    outbuf[j++] = (c & 0xff);
+	    outbuf[j++] = (c >> 8);
 
 	}
-	return file->write(file, outbuf, count*2);
+	return file->write(file, outbuf, j);
 
     default:
       ERR2("Bad output character encoding %d (%s)\n",
@@ -307,6 +385,13 @@ int Fclose(FILE16 *file)
 
     ret = file->close(file);
     Free(file);
+
+    if(file == Stdin)
+	Stdin_open = 0;
+    else if(file == Stdout)
+	Stdout_open = 0;
+    else if(file == Stderr)
+	Stderr_open = 0;
     
     return ret;
 }
@@ -649,6 +734,24 @@ FILE16 *MakeFILE16FromFILE(FILE *f, const char *type)
     if(!(file = MakeFILE16(type)))
 	return 0;
 
+#ifdef WIN32
+    /* We need to do Windows-style lf <-> crlf conversion ourselves, 
+       because the standard i/o library doesn't know about 16-bit 
+       characters. We don't want it to insert cr before output bytes 
+       that just happen to look like linefeed, or change input byte
+       sequences that happen to look like crlf. */
+
+    file->flags |= FILE16_crlf;
+
+    /* So we have to switch the underlying file into binary mode, but
+       we probably don't want to automatically do that for Stdin/out/err
+       which are opened though the user may not want them (and may
+       want to keep stdin/out/err in text mode).  So we just flag
+       it as needing to be switched when the first i/o is done. */
+
+    file->flags |= FILE16_needs_binary;
+#endif
+
     file->read = FileRead;
     file->write = FileWrite;
     file->seek = FileSeek;
@@ -664,6 +767,14 @@ static int FileRead(FILE16 *file, unsigned char *buf, int max_count)
     FILE *f = file->handle;
     int count = 0;
 
+#ifdef WIN32
+    if(file->flags & FILE16_needs_binary)
+    {
+	_setmode(fileno(f), _O_BINARY);
+	file->flags &= ~FILE16_needs_binary;
+    }
+#endif
+
     count = fread(buf, 1, max_count, f);
 
     return ferror(f) ? -1 : count;
@@ -673,6 +784,14 @@ static int FileWrite(FILE16 *file, const unsigned char *buf, int count)
 {
     FILE *f = file->handle;
 
+#ifdef WIN32
+    if(file->flags & FILE16_needs_binary)
+    {
+	_setmode(fileno(f), _O_BINARY);
+	file->flags &= ~FILE16_needs_binary;
+    }
+#endif
+
     if(count == 0)
 	return 0;
     return fwrite(buf, 1, count, f) == 0 ? -1 : 0;
@@ -799,6 +918,8 @@ FILE16 *MakeFILE16FromWinsock(int sock, const char *type)
     if(!(file = MakeFILE16(type)))
 	return 0;
 
+    file->flags |= FILE16_crlf;
+
     file->read = WinsockRead;
     file->write = WinsockWrite;
     file->seek = WinsockSeek;
@@ -834,7 +955,10 @@ static int WinsockSeek(FILE16 *file, long offset, int ptrname)
 
 static int WinsockClose(FILE16 *file)
 {
-    /* What should happen here? XXX */
+    int f = file->handle2;
+
+    if(file->flags & FILE16_close_underlying)
+	closesocket(f);
 
     return 0;
 }
