@@ -131,8 +131,12 @@ int init_url(void)
 
     if((p = getenv("http_proxy")))
     {
-	char *colon;
+	char *colon, *slash;
+	if(strncmp(p, "http://", 7) == 0)
+	    p += 7;
 	proxy_host = strdup8(p);
+	if((slash = strchr(proxy_host, '/')))
+	    *slash = 0;
 	if((colon = strchr(proxy_host, ':')))
 	{
 	    proxy_port = atoi(colon+1);
@@ -145,57 +149,94 @@ int init_url(void)
     return 0;
 }
 
-/* Construct a default base URL, essentially file:`pwd`/ */
+void deinit_url(void)
+{
+#ifdef WIN32
+    /* No doubt there's some Windozy nonsense that should go here,
+       but I have no idea what it is. */
+#endif
+
+    if(proxy_host)
+	Free(proxy_host);
+}
+
+/* Construct a default base URL, essentially file://`pwd`/ */
 
 char *default_base_url(void)
 {
     char buf[CWDBS];
     char *url;
+    int len;
     
     if(!GETWD(buf))
     {
 	WARN(LEFILE, "Warning: can't get current directory for default base url\n");
-	return strdup8("file:/");
+	return strdup8("file:///");
     }
 
+    len = strlen(buf);
 
 #ifdef WIN32
 
-    /* DOS: translate C:\a\b to file:/C:/a/b/ */
+    /* DOS: translate C:\a\b to file:///C:/a/b/ */
     /* XXX should we escape anything? */
     {
-    char *p;
-    for(p=buf; *p; p++)
-	if(*p == '\\')
-	    *p = '/';
+	char *p;
+	for(p=buf; *p; p++)
+	    if(*p == '\\')
+		*p = '/';
+
+	if(buf[len-1] == '/')
+	    /* happens for C:\ etc */
+	    buf[--len] = 0;
     }
-    url = Malloc(6 + strlen(buf) + 2);
-    sprintf(url, "file:/%s/", buf);
+    url = Malloc(8 + len + 2);
+    sprintf(url, "file:///%s/", buf);
 
 #else
 #ifdef mac_filenames
 
-    /* Mac: translate a:b to file:/a/b/ */
+    /* Mac: translate a:b to file:///a/b/ */
     /* XXX should escape spaces and slashes, at least */
     {
 	char *p;
 	for(p=buf; *p; p++)
 	    if(*p == ':')
 		*p = '/';
-	/* Mac getcwd (always?) has a trailing separator, which we here bash */
-	if(*--p == '/') 
-	    *p = 0;
+
+	if(buf[len-1] == '/')
+	    /* often (always?) has trailing separator */
+	    buf[--len] = 0;
     }
-    url = Malloc(6 + strlen(buf) + 2);
-    sprintf(url, "file:/%s/", buf);
+    url = Malloc(8 + len + 2);
+    sprintf(url, "file:///%s/", buf);
 
 #else
+#ifdef CYGNUS32
 
-    /* Unix: translate /a/b to file:/a/b/ */
+    /* Cygnus: translate //c/a/b to file:///c:/a/b  */
 
-    url = Malloc(5 + strlen(buf) + 2);
-    sprintf(url, "file:%s/", buf);
+    if(buf[len-1] == '/')
+	/* should only happen for root directory */
+	buf[--len] = 0;
 
+    url = Malloc(8 + len + 2);
+    if(buf[0] == '/' && buf[1] == '/')
+	sprintf(url, "file:///%c:%s/", buf[2], buf+3);
+    else
+	sprintf(url, "file://%s/", buf);
+#else
+
+    /* Unix: translate /a/b to file:///a/b/ */
+
+    if(buf[len-1] == '/')
+	/* should only happen for root directory */
+	buf[--len] = 0;
+
+    url = Malloc(7 + len + 2);
+    sprintf(url, "file://%s/", buf);
+
+#endif
 #endif
 #endif
 
@@ -388,7 +429,7 @@ bad:
  * Open a stream to a URL.
  * url may be a relative URL, in which case it is merged with base,
  * which is typically the URL of the containing document.  If base
- * is null, file:`pwd`/ is used, which is the right thing to do for
+ * is null, file://`pwd`/ is used, which is the right thing to do for
  * filenames.  If base is "", there is no base URL and relative
  * URLs will fail.
  * If merged_url is non-null the resulting URL is stored in it.
@@ -762,12 +803,26 @@ static FILE16 *file_open(const char *url,
 	    if(*p == '/')
 		*p = ':';
     }
+#else
+#ifdef CYGNUS32
+
+    /* Cygnus: translate /c:/a/b.c to //c/a/b.c */
+
+    file = strdup8(path);
+
+    if(path[0] == '/' && path[1] && path[2] == ':')
+    {
+	file[0] = file[1] = '/';
+	file[2] = path[1];
+    }
+
 #else
 
     /* Unix: a path is a path is a path! */
 
     file = strdup8(path);
 
+#endif
 #endif
 #endif
 
