@@ -1,4 +1,4 @@
-/* 	$Id: xmlparser.h,v 1.38 2000/01/27 16:49:49 richard Exp $    */
+/* 	$Id: xmlparser.h,v 1.39 2000/10/26 15:13:50 richard Exp $    */
 
 #ifndef XMLPARSER_H
 #define XMLPARSER_H
@@ -95,12 +95,10 @@ struct xbit {
 enum parser_flag {
     ExpandCharacterEntities,
     ExpandGeneralEntities,
-    XMLPiEnd,
-    XMLEmptyTagEnd,
+    XMLSyntax,
     XMLPredefinedEntities,
     ErrorOnUnquotedAttributeValues,
     NormaliseAttributeValues,
-    NormalizeAttributeValues,
     ErrorOnBadCharacterEntities,
     ErrorOnUndefinedEntities,
     ReturnComments,
@@ -124,10 +122,14 @@ enum parser_flag {
     XMLSpace,
     XMLNamespaces,
     NoNoDTDWarning,
-    SimpleErrorFormat
+    SimpleErrorFormat,
+    AllowUndeclaredNSAttributes,
+    RelaxedAny
 };
 typedef enum parser_flag ParserFlag;
 
+#define NormalizeAttributeValues NormaliseAttributeValues
+
 /* Parser */
 
 enum parse_state 
